"""
Classifies: CHEBI:23424 cyanides
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_cyanides(smiles: str):
    """
    Determines if a molecule contains a cyanide group (C#N).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule contains cyanide group, False otherwise
        str: Reason for classification
    """
    # Parse SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # SMARTS pattern for cyanide group (C#N)
    cyanide_pattern = Chem.MolFromSmarts('C#N')
    
    # Find matches
    matches = mol.GetSubstructMatches(cyanide_pattern)
    
    if not matches:
        return False, "No cyanide (C#N) group found"
        
    # Get all cyanide carbons
    cyanide_carbons = [match[0] for match in matches]
    
    # Get all cyanide nitrogens 
    cyanide_nitrogens = [match[1] for match in matches]
    
    # Verify triple bond between C and N
    for c, n in zip(cyanide_carbons, cyanide_nitrogens):
        bond = mol.GetBondBetweenAtoms(c, n)
        if bond.GetBondType() != Chem.BondType.TRIPLE:
            return False, "C-N bond is not a triple bond"
            
    # Count number of cyanide groups
    num_cyanides = len(matches)
    
    # Get all atoms connected to cyanide carbons
    substituents = []
    for c in cyanide_carbons:
        atom = mol.GetAtomWithIdx(c)
        for neighbor in atom.GetNeighbors():
            if neighbor.GetIdx() not in cyanide_nitrogens:
                substituents.append(neighbor.GetSymbol())
                
    return True, f"Contains {num_cyanides} cyanide group(s) with substituents: {', '.join(set(substituents))}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23424',
                          'name': 'cyanides',
                          'definition': 'Salts and C-organyl derivatives of '
                                        'hydrogen cyanide, HC#N.',
                          'parents': ['CHEBI:35352']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 66,
    'num_false_positives': 100,
    'num_true_negatives': 7178,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.39759036144578314,
    'recall': 1.0,
    'f1': 0.5689655172413793,
    'accuracy': 0.9863834422657952}


"""
Classifies: CHEBI:24676 hydroxybenzoic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_hydroxybenzoic_acid(smiles: str):
    """
    Determines if a molecule is a hydroxybenzoic acid (benzoic acid with one or more phenolic hydroxy groups).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a hydroxybenzoic acid, False otherwise
        str: Reason for classification
    """
    # Parse SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Look for carboxylic acid groups attached to aromatic rings or within a lactone
    carboxylic_patterns = [
        Chem.MolFromSmarts('[OH]-[C](=[O])-[cr5,cr6]'),  # Include both 5 and 6 membered rings
        Chem.MolFromSmarts('[O-]-[C](=[O])-[cr5,cr6]'),
        Chem.MolFromSmarts('O=[C]([O-])-[cr5,cr6]'),
        Chem.MolFromSmarts('O=[C](O)-[cr5,cr6]'),
        Chem.MolFromSmarts('O=C1O[C@@H](*)*C=2C1=C(*)*C=C(*)*2'),  # Lactone pattern
        Chem.MolFromSmarts('O=C1O[C@H](*)*C=2C1=C(*)*C=C(*)*2'),   # Lactone pattern
        Chem.MolFromSmarts('O=C1OC(*)*C=2C1=C(*)*C=C(*)*2')        # Generic lactone pattern
    ]
    
    found_carboxylic = False
    ring_carbons = set()
    
    for pattern in carboxylic_patterns:
        if pattern is None:
            continue
        matches = mol.GetSubstructMatches(pattern)
        if matches:
            found_carboxylic = True
            for match in matches:
                # Get the ring carbon - last atom in the match for direct acids,
                # or part of the ring system for lactones
                ring_carbons.update([i for i in match if mol.GetAtomWithIdx(i).IsInRing()])
            
    if not found_carboxylic:
        return False, "No carboxylic acid group found"

    # Look for hydroxy groups attached to the rings containing carboxylic groups
    hydroxy_patterns = [
        Chem.MolFromSmarts('[OH]-[cr5,cr6]'),
        Chem.MolFromSmarts('[O-]-[cr5,cr6]')
    ]
    
    phenolic_oh_count = 0
    for pattern in hydroxy_patterns:
        if pattern is None:
            continue
        matches = mol.GetSubstructMatches(pattern)
        for match in matches:
            ring_atom = match[1]
            # Check if the OH is attached to a ring that contains a carboxylic group
            for ring in mol.GetRingInfo().AtomRings():
                if ring_atom in ring and any(c in ring for c in ring_carbons):
                    phenolic_oh_count += 1
                    break
                
    if phenolic_oh_count > 0:
        return True, f"Found benzoic acid with {phenolic_oh_count} phenolic OH group(s)"
    else:
        return False, "No phenolic OH groups found on the ring system"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24676',
                          'name': 'hydroxybenzoic acid',
                          'definition': 'Any benzoic acid carrying one or more '
                                        'phenolic hydroxy groups on the '
                                        'benzene ring.',
                          'parents': ['CHEBI:22723', 'CHEBI:33853']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.2682926829268293 is too low.\n'
               'True positives: '
               "[('O1C(CCC=C(C)C)(C=CC=2C1=C(O)C=C(O)C2C(O)=O)C', 'Found "
               "benzoic acid with 2 phenolic OH group(s)'), "
               "('O=C1C2=C(OC3=C1CO[C@@H](C3)C)C(O)=C(OC)C=C2C(=O)OC', 'Found "
               "benzoic acid with 1 phenolic OH group(s)'), "
               "('CC1=C(C(O)=O)C(O)=CC(O)=C1', 'Found benzoic acid with 2 "
               "phenolic OH group(s)'), ('OC(=O)c1cccc(C(O)=O)c1O', 'Found "
               "benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C(O)C1=C(O)C=C(O)C=C1C(=O)C(=O)C', 'Found benzoic acid "
               "with 2 phenolic OH group(s)'), "
               "('[H][C@@]1(O[C@@](CC)(C[C@@H]1C)[C@@]1([H])CC[C@](O)(CC)[C@H](C)O1)[C@@H](CC)C(=O)[C@@H](C)[C@@H](O)[C@H](C)CCC1=C(C(O)=O)C(O)=C(C)C=C1', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('C1=2C(=C(C=C3C(N(C(C(=CC=C1)C23)=O)C=4C=CC(=C(C4)C(=O)O)O)=O)S(O)(=O)=O)N', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C1C(OC(=O)C2=C(O)C=C(O)C=C2C)=CO[C@H](C1)C', 'Found "
               "benzoic acid with 2 phenolic OH group(s)'), "
               "('C1=CC(=C(C=C1C(C(=O)O)N)C(=O)O)O', 'Found benzoic acid with "
               "1 phenolic OH group(s)'), "
               "('OC=1C(=C(CCCCCCCCC/C=C\\\\CCCC)C=CC1)C(O)=O', 'Found benzoic "
               "acid with 1 phenolic OH group(s)'), "
               "('O=C1NC2=C(O)C(=CC(=C2)O)[C@@H](NC3=CC(O)=CC(=C3)C(=O)O)CC=CC=C[C@H]([C@H]1C)O', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('ClC1=C(O)C=C(O)C2=C1CC(=O)CCC(O)C[C@@H]3[C@H](C[C@H](OC2=O)C)O3', "
               "'Found benzoic acid with 2 phenolic OH group(s)'), "
               "('O=C1N(CC=2C1=C(C(O)=C(CCC(=O)/C=C/C=C/CCCCCCC)C2O)C(=O)O)CCC(C)C', "
               "'Found benzoic acid with 2 phenolic OH group(s)'), "
               "('C(=O)(C1=CC(=C(C=C1)O)O)O', 'Found benzoic acid with 2 "
               "phenolic OH group(s)'), "
               "('[H][C@@]12CC[C@@]3(C)[C@H](Cc4cc(ccc4O)C(O)=O)C(C)=CC[C@]3([H])[C@@]1(C)CCCC2(C)C', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('CC(C)=CCC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\Cc1cc(cc(O)c1O)C(O)=O', "
               "'Found benzoic acid with 2 phenolic OH group(s)'), "
               "('Cc1c(O)cc(O)c2C(=O)OC(=C)[C@](C)(O)c12', 'Found benzoic acid "
               "with 2 phenolic OH group(s)'), "
               "('ClC1=C(O)C=C(O)C2=C1[C@@H](Cl)[C@@H](C)OC2=O', 'Found "
               "benzoic acid with 2 phenolic OH group(s)'), "
               "('O=C1O[C@H](C2=C(O)C=C(C(=O)O)C=C2O)C=3C1=C(O)C=C(O)C3', "
               "'Found benzoic acid with 2 phenolic OH group(s)'), "
               "('C12=C(NCC(=N1)CNC3=CC=C(C(O)=O)C(=C3)O)N=C(N)NC2=O', 'Found "
               "benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C1O[C@H](O)C=2C1=C(OC)C(=C(O)C2C(=O)O)C', 'Found benzoic "
               "acid with 1 phenolic OH group(s)'), "
               "('OC=1C(=C(CCCCCC/C=C\\\\CCCCC)C=CC1)C(O)=O', 'Found benzoic "
               "acid with 1 phenolic OH group(s)')]\n"
               'False positives: '
               "[('O1C(C(O)C(O)C(O)C1OC2=CC(O)=C(C=C2)C(O)=O)C(O)=O', 'Found "
               "benzoic acid with 1 phenolic OH group(s)'), "
               "('COC(=O)c1c(O)cc2cc3C(=O)c4cc(OC)cc(O)c4C(=O)c3c(O)c2c1C', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('C=1(C=CC(=C(C1)Br)O)C([O-])=O', 'Found benzoic acid with 1 "
               "phenolic OH group(s)'), "
               "('O=C(OC1=C(C(O)=C(C)C(=C1)C)C)C2=C(O)C=C(O)C=C2C', 'Found "
               "benzoic acid with 2 phenolic OH group(s)'), "
               "('[Na+].[Na+].[Na+].Oc1ccc(cc1C([O-])=O)C(=C1C=CC(=O)C(=C1)C([O-])=O)c1ccc(O)c(c1)C([O-])=O', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C(OC)C1=CC(O)=C([C@](O)(CCCC(O)(C)C)C)C=C1', 'Found "
               "benzoic acid with 1 phenolic OH group(s)'), "
               "('O1[C@@H]([C@@H](O)[C@H](O)[C@@H](O)[C@@H]1OC2=C(O)C=C(C=C2)COC(=O)C3=CC(O)=C(O)C=C3)CO', "
               "'Found benzoic acid with 2 phenolic OH group(s)'), "
               "('Cc1cc(O)c(O)c(Oc2cc(C)c(C(O)=O)c(O)c2)c1', 'Found benzoic "
               "acid with 1 phenolic OH group(s)'), "
               "('[H][C@]12C[C@]3([H])O[C@@]1(C)[C@H](O)[C@@]1(C2)C=CC(=O)[C@@](C)(CCC(=O)Nc2c(O)ccc(C(=O)OC)c2O)[C@]31[H]', "
               "'Found benzoic acid with 2 phenolic OH group(s)'), "
               "('O=C(OC)C1=C(O)C2=C(OC(C)=C[C@@]2([C@@H]3[C@@]4(OC=5C=C(C)C(=C(C5[C@H]3C=6C(O)=CC(=CC6O4)C)O)C(=O)OC)C)C)C=C1C', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C(OCC1=CC(O)=C([C@@](O)(CCCC(C)C)C)C=C1)C2=CC(O)=C([C@@](O)(CCCC(C)C)C)C=C2', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('CC(=O)OC1=C(C=C(C=C1)C(=O)NC2=C(C=C(C=C2)O)C(=O)O)OC', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C(OC1=C(C(O)=C(C(=O)OC)C(=C1C)C)C)C2=C(O)C(=C(O)C(=C2C)C)C=O', "
               "'Found benzoic acid with 2 phenolic OH group(s)'), "
               "('ClC1=C(O)C(=C(O)C(=C1C)C(=O)O)C/C=C(/CCC[C@](O)(CCC=C(C)C)C)\\\\C', "
               "'Found benzoic acid with 2 phenolic OH group(s)'), "
               "('O=C(OCC)C1=C(O)C=C(OC)C=C1CCC', 'Found benzoic acid with 1 "
               "phenolic OH group(s)'), "
               "('CC(C)=CCC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\Cc1cc(cc(O)c1O)C([O-])=O', "
               "'Found benzoic acid with 2 phenolic OH group(s)'), "
               "('O=C1OC(=CC=2C1=C(O)C(=C(O)C2)C(=O)C3=C(O)C=C(OC)C=C3C(=O)OC)C', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C(OC1=CC(O)=CC(=C1)CCC)C2=C(O)C=C(OC)C=C2CCC', 'Found "
               "benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C(OC)C1=CC(O)=C([C@@]2(OC(CCC2)(C)C)C)C=C1', 'Found "
               "benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C(O[C@H]1[C@H]2C(=C[C@@H]3C[C@@](CC3[C@@]2(C)C1)(CO)C)CO)C4=C(O)C=C(O)C=C4C', "
               "'Found benzoic acid with 2 phenolic OH group(s)'), "
               "('O=C(OC1=C(O)C(=C(CCC)C=C1OC)C(=O)O)C2=C(O)C=C(OC)C=C2CCC', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C(OC1=C(C(O)=C(C(=O)O)C(=C1)C)C)C2=C(OC)C(=C(OC(=O)C3=C(O)C=C(O)C=C3C)C=C2C)C', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O(C1=CC(=CC(O)=C1)C(O)=O)C', 'Found benzoic acid with 1 "
               "phenolic OH group(s)'), "
               "('ClC1=C(O)C(=C(C(=O)OC)C=C1)C(=O)C2=C(O)C=C(C)C=C2O', 'Found "
               "benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C(OC1=CC(O)=C(C(=O)O)C(=C1)CCC)C2=C(O)C=C(OC)C=C2CCCCC', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('ClC1=C(O)C=C(OC)C2=C1CC(=O)CCCC[C@@H](CC[C@@H](OC2=O)C)O', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O=[N+]([O-])C1=C(C(OC2=C(C(O)=CC(=C2)C)C(=O)OC[C@@H](C(=O)O)C)=C(OC)C=C1O)C(=O)OC', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('C=1(C(=CC(O)=CC1)C([O-])=O)NC(/C=C/C2=CC=C(O)C=C2)=O', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('ClC1=C(OC2=C(OC)C=C(OC)C=C2C)C(=C(O)C=C1OC)C(=O)OC', 'Found "
               "benzoic acid with 1 phenolic OH group(s)'), "
               "('O(C=1C(=C(O)C=CC1)C(O)=O)C', 'Found benzoic acid with 1 "
               "phenolic OH group(s)'), "
               "('O=C1N([C@H](C(=O)N[C@@H](C(=O)N[C@@H](C)CC(O[C@@H]([C@@H](C(N[C@H]1[C@H](CC)C)=O)NC(=O)C(=NNC2=C(C(=O)OCCCCCCCCCC(C)C)C=C(O)C=C2)C)C3=CC=CC=C3)=O)CC=4C5=C(C(OC)=CC=C5)NC4)[C@@H](O)C6=CC=CC=C6)C', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O1[C@@H](O[C@@H]2OC=C3[C@]([C@H]2C=C)(CCOC3=O)[H])[C@H](OC(=O)C)[C@@H](OC(=O)C)[C@H](OC(=O)C4=C(O)C(O[C@@H]5O[C@@H]([C@@H](O)[C@H](O)[C@H]5O)CO)=CC=C4)[C@H]1COC(=O)C', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('CCN(CC)S(=O)(=O)C1=CC(=C(C=C1)O)C(=O)OCC(=O)NC2=CC=C(C=C2)OC(F)(F)F', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C(OC1=C(O)C(=C(O)C=C1C)C)C2=C(O)C(=C(OC)C=C2C)C(=O)O', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('CC(C)=CCC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\COC(=O)c1ccc(O)cc1', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C1C2=C(C(O)=CC=C2OC3=C1C(OC)=CC(=C3)C)C(=O)OC', 'Found "
               "benzoic acid with 1 phenolic OH group(s)'), "
               "('ClC1=C(C(=C(O)C=C1)C(=O)O[C@H]2[C@H](O)[C@@H](O)[C@H](O[C@@H]3[C@@H](O)[C@@H](O[C@@H]([C@H]3O)C)O[C@@H]4[C@H]5C=CC6[C@@H](C=CC[C@@]7(C=C(C)[C@@H](C[C@]87C(=C(OC([C@]6([C@@H]5[C@@H](C)C[C@H]4C)C)=O)C(=O)O8)O)C[C@H](O)[C@@](O)(C(=O)C)C)C)C)O[C@@H]2C)C', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('CCN(CC)S(=O)(=O)C1=CC=CC2=CC(=C(C=C21)O)C(=O)O', 'Found "
               "benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C1C2=C(C(=O)C=3C1=C(O)C(=C(C)C3)C(=O)O)C(O)=C(O)C=C2O', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('ClC1=C(O)C=C(OC)C2=C1CC(=O)CCCCC(CC[C@@H](OC2=O)C)=O', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('C1=CC(=CC=C1C(=O)OCC2=CC=CC=C2)O', 'Found benzoic acid with "
               "1 phenolic OH group(s)'), "
               "('O=C(O)C1=C(O)C=C(O)C=C1C2=C(C=C(O)C(=C2)O)C', 'Found benzoic "
               "acid with 2 phenolic OH group(s)'), "
               "('O=C(O)C1=CC(O)=C(/C(=C/CC[C@@H](COC(=O)C)C)/C)C=C1', 'Found "
               "benzoic acid with 1 phenolic OH group(s)'), "
               "('S1C2=C(C3=C(C(O)=CC(=C3)OC)C(=O)O)C(=CC(=C2N=C1)O)C', 'Found "
               "benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C(OC)C1=C(C(O)=CC(=C1)OC)C(=O)C2=C(O)C=C(C)C3=C2O[C@@](C=C=C4[C@@H](O)[C@]5(O[C@H]5[C@H](C4)O)CC=C(C)C)(C)C3', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C1OC2=C(C(=C(C(=O)O)C(=C2C)O)C(=CC)C)OC3=C1C(C(=CC)C)=CC(=C3C)O', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C(O)C1=C(O)C(C(=O)[C@@H]([C@H](O)C2=C(OC)C=CC(=C2)OC)C)=CC=C1OC', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O1[C@@]2([C@]1(C#CC=C3[C@@H](O)[C@H](OC(=O)C=4C5=C(C(=CC(OC)=C5)C)C=CC4O)C=C3C#C2)[H])[C@@]6(OC(OC6)=O)[H]', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('[O-]C(=O)c1cc(Br)c([O-])c(Br)c1', 'Found benzoic acid with 1 "
               "phenolic OH group(s)'), "
               "('Cc1cc2[C@H](O)[C@@H](O)c3c(O)c4C(=O)c5c(O)cc(O)cc5C(=O)c4c(O)c3-c2c(O)c1C(O)=O', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C1O[C@](OC)(CC=2C1=C(O)C=3C4=C(O)C=5C(=O)C6=C(O)C(C(=O)O)=CC=C6OC5C(=C4CCC3C2)O)C', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('OC(=O)c1c(O)[nH]c2ccccc2c1=O', 'Found benzoic acid with 1 "
               "phenolic OH group(s)'), "
               "('ClC1=C(O)C=C(O)C(=C1C)C(=O)O[C@H]2[C@@]3(O)C(=C[C@]4(O)CC(C[C@@H]4[C@@]3(C)C2)(C)C)C=O', "
               "'Found benzoic acid with 2 phenolic OH group(s)'), "
               "('O=C(N1C(CCC1)C(=O)NC2=C(C=CC=C2O)C(O)=O)C(NC(=O)C3=CC=C(O)C=C3)C(C)C', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O[C@@H]1[C@H]2COC(=O)c3cc(O)c(O)c(O)c3-c3c(O)c(O)c(Oc4c(O)c(O)c(O)cc4C(O)=O)cc3C(=O)O[C@@H]1[C@@H](O)[C@H](OC(=O)c1cc(O)c(O)c(O)c1)O2', "
               "'Found benzoic acid with 3 phenolic OH group(s)'), "
               "('OC(=O)C(=O)CCc1c(O)cc([nH]c1=O)C(O)=O', 'Found benzoic acid "
               "with 1 phenolic OH group(s)'), "
               "('O=C1C(C2=C(C(OC)=CC(=C2)O)C(=O)O)=C(C)C[C@H]1O', 'Found "
               "benzoic acid with 1 phenolic OH group(s)'), "
               "('Oc1ccc(Cl)cc1C([O-])=O', 'Found benzoic acid with 1 phenolic "
               "OH group(s)'), "
               "('O=C(O[C@H]1[C@H](O)[C@@H](O)[C@H](OC(=O)C2=C(O)C=CC=C2C)[C@H](C1)NC(=O)C3=CC(O)=CC=C3)N', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('ClC1=C(O)C(Cl)=C(C)C(=C1O)C(=O)O', 'Found benzoic acid with "
               "2 phenolic OH group(s)'), "
               "('O.O.[Na+].[Na+].Oc1ccc(cc1C([O-])=O)\\\\N=N\\\\c1ccc(cc1)C(=O)NCCC([O-])=O', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C1C(OC(=O)C2=C(O)C=C(O)C=C2C)C(OC(C1O)C)OC', 'Found "
               "benzoic acid with 2 phenolic OH group(s)'), "
               "('[H][C@]1(C=C(C)CC[C@H]1C(C)=C)c1c(O)cc(CCCCC)c(C([O-])=O)c1O', "
               "'Found benzoic acid with 2 phenolic OH group(s)'), "
               "('Cc1c(O)c2C(=O)C[C@H](Oc2c(C)c1O[C@@H]1O[C@H](COC(=O)c2ccc(O)cc2)[C@@H](O)[C@H](O)[C@H]1O)c1cc(O)ccc1O', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C(O)C=1C2=NC=CC=C2C=C(C1)O', 'Found benzoic acid with 1 "
               "phenolic OH group(s)'), "
               "('O=C(O)C1=CC(O)=C([C@@](O)(CCCC(=C)C)C)C=C1', 'Found benzoic "
               "acid with 1 phenolic OH group(s)'), "
               "('O[C@@]1([C@H]2[C@](CC1)(CC=C(C[C@@H]2OC(=O)C3=CC=C(O)C=C3)C)C)C(C)C', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('ClC1=C(O)C(Cl)=CC(=C1)C(=O)OC', 'Found benzoic acid with 1 "
               "phenolic OH group(s)'), "
               "('[C@@H]1(N2C3=C(C(=NC=N3)N)N=C2)O[C@H](COP(OP(OCC(C)([C@H](C(NCCC(NCCSC(=O)C[C@H](\\\\C(=C\\\\CC\\\\C(=C\\\\CC4=C(C(=C5C(=C4O)C(OC5)=O)C)OC)\\\\C)\\\\C)O)=O)=O)O)C)(=O)[O-])(=O)[O-])[C@H]([C@H]1O)OP([O-])([O-])=O', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('ClC1=C(OC)C(=C(C)C(=C1O)Cl)C(=O)O[C@H]2[C@H](O[C@@H]3O[C@H]([C@H](OC)[C@](C3)([N+](=O)[O-])C)C)C[C@H](O[C@H]4[C@H](O)C[C@@]5(O[C@@]6([C@H](O)[C@H](O[C@H]7[C@@H](OC)[C@H](O[C@H]([C@@H]7O)O[C@H]8[C@H](O)[C@H](OC)[C@H](O[C@@H]9OC[C@@H]%10O[C@@]%11(O[C@H]%10[C@H]9O)OC[C@@H](OC(=O)C%12=C(O)C=C(O)C=C%12C)[C@H]%13[C@H]%11OCO%13)O[C@@H]8COC)C)O[C@@H]([C@H]6O5)C)C)O[C@@H]4C)O[C@@H]2C', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('[H][C@]12CC[C@@]3(C=CC(=O)[C@@](C)(CCC(=O)Nc4c(O)ccc(C(=O)OC)c4O)[C@]3([H])C1)[C@@H](O)C2=C', "
               "'Found benzoic acid with 2 phenolic OH group(s)'), "
               "('C1=C([C@@]2([C@@H](C[C@@]2([C@@]3([C@]1(CC(C3)(C)C)[H])[H])C)OC(C4=C(C(=C(C=C4O)O)Br)C)=O)O)CO', "
               "'Found benzoic acid with 2 phenolic OH group(s)'), "
               "('C1(=C(C(=CC(=C1)O)O)C([O-])=O)/C=C/CCCC(=O)CCC[C@@H](O)C', "
               "'Found benzoic acid with 2 phenolic OH group(s)'), "
               "('[Cl-].C1=2C=CC(=CC1=[O+]C3=C(N2)C(=CC(=C3O)O)C(=O)O)N(C)C', "
               "'Found benzoic acid with 2 phenolic OH group(s)'), "
               "('O=C1C(=N)C(=C(O)C(=C1)C2=C(O)C(=C(N)C(=C2)OC)C(=O)O)C(=O)O', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C(O)C1=CC(OC2=CC(O)=CC(=C2)C)=CC(=C1)O', 'Found benzoic "
               "acid with 1 phenolic OH group(s)'), "
               "('O=C(O)C1=C(O)C2=C(OC=3C=C(C)C=C(C3[C@@H]2C=C(C)C)O)C=C1C', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O[C@@H]1[C@@H](COC(=O)c2cc(O)c(O)c(O)c2)O[C@@H](OC(=O)c2cc(O)c(O)c(O)c2)[C@H](OC(=O)c2cc(O)c(O)c(O)c2)[C@H]1OC(=O)c1cc(O)c(O)c(O)c1', "
               "'Found benzoic acid with 3 phenolic OH group(s)'), "
               "('O1[C@@H]([C@H](OC(=O)C2=CC(OC)=C(O)C(OC)=C2)CC=3C1=CC(O)=CC3O)C4=CC(O)=C(O)C(O)=C4', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C(O)C1=C2O[C@]3(OC)[C@H](CCCC3)[C@@H]4C2=C([C@@H]([C@@H](C)O4)C)C(=C1O)C', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('CCN(C1=CC=CC=C1)S(=O)(=O)C2=CC(=C(C=C2)Cl)NC(=O)COC(=O)C3=C(C(=CC=C3)OC)O', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C1C2=C(CC(/C=C/C)OC2)C[C@@H]([C@@]1(OC(=O)C3=C(O)C=C(O)C=C3C)C)OC(=O)C', "
               "'Found benzoic acid with 2 phenolic OH group(s)'), "
               "('O(C1CC2(C3C(C4(C(C5C(CC4)(CCC5C(C)=C)C(O)=O)CC3)C)(CCC2C(C1O)(C)C)C)C)C(=O)C6=CC(O)=C(O)C=C6', "
               "'Found benzoic acid with 2 phenolic OH group(s)'), "
               "('O=C(O)C1=C(O)C2=C(C[C@@H](CCCCCO)O[C@@H]2OC)C=C1O', 'Found "
               "benzoic acid with 2 phenolic OH group(s)'), "
               "('OC(=O)c1ncccc1O', 'Found benzoic acid with 1 phenolic OH "
               "group(s)'), "
               "('C\\\\C=C\\\\C1=CC2=CC(=O)C(C)(OC(=O)c3c(C)cc(O)cc3O)C(=O)C2=CO1', "
               "'Found benzoic acid with 2 phenolic OH group(s)'), "
               "('C1=2C(=C(C=C3C(N(C(C(=CC=C1)C23)=O)C=4C=CC(=C(C4)C(=O)[O-])O)=O)S([O-])(=O)=O)N', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('Cc1cccc2c(cc(O)cc12)C([O-])=O', 'Found benzoic acid with 1 "
               "phenolic OH group(s)'), "
               "('ClC1=C(O)C=C(OC2=C(O)C(=C(O)C=C2/C(=C/C)/C)C)C(=C1C)C(=O)O', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O1[C@@H]([C@H](OC(=O)C=2C=C(OC)C(O)=C(O)C2)CC=3C1=CC(O)=CC3O)C4=CC(O)=C(O)C(O)=C4', "
               "'Found benzoic acid with 2 phenolic OH group(s)'), "
               "('C=12C(=C3C(=C(C(=CC3=CC1C[C@H](OC2=O)C[C@@H](C[C@@H](CCCCCO)O)O)OC)C4=C(C5=C(C=6C(O[C@@H](CC6C=C5C=C4OC)C[C@@H](C[C@@H](CCCCCO)O)O)=O)O)O)O)O', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('S1C2=C(C(C3=C(C(O)=CC(=C3)OC)C(=O)O)=CC(=C2N=C1)O)C', 'Found "
               "benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C(OCC1=C[C@H]2[C@H]([C@@H](O)C(C2)(C)C)[C@@]3([C@@]1(O)[C@H](O)C3)C)C4=C(O)C=C(OC)C=C4C', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O(C1(C(OC=2C(=C(O)C=C(C2C(OC)=O)C)C)C(=O)C(=C(O)C1O)C)C)C', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C1OC23C(C=C(CCC=C(C4C(C(=C1C2=O)O)(C5C=C(CC(C5CC4C)OC6OC(C(OC(=O)C7=C(OC)C=C(O)C=C7C)C(C6O)O)C)C)C)C)C)(C=C(C(=O)O)C(C3)C)C', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C(O[C@H](CC(=O)O)C)C1=C(O)C=C(O)C=C1C[C@H](OC(=O)C[C@H](OC(=O)C[C@@H](O)CO)C)C', "
               "'Found benzoic acid with 2 phenolic OH group(s)'), "
               "('O=C(OC1=C(O)C(=C(OC(=O)C2=CC=C(O)C=C2)C(=C1C3=CC=C(O)C=C3)O)C4=CC=C(O)C=C4)C5=CC=C(O)C=C5', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C(O)C1=CC(=C(O)C=C1)C/C=C/C(C/C=C/C(C/C=C/C(CC/C=C(\\\\C(=O)O)/C)C)C)C', "
               "'Found benzoic acid with 1 phenolic OH group(s)'), "
               "('O=C(O)C1=C(O)C2=C(C[C@@H](CCCCCCC)OC2)C(=C1O)C', 'Found "
               "benzoic acid with 2 phenolic OH group(s)'), "
               "('O=C1C=C2C=C(OC[C@H]2[C@@H]([C@@]1(OC(=O)C3=C(O)C=C(O)C=C3C)C)O)C=CCO', "
               "'Found benzoic acid with 2 phenolic OH group(s)')]\n"
               "False negatives: [('O=C1O[C@](OC)(CC=2C1=C(O)C(=C(O)C2C)C)C', "
               "'Ring is not aromatic'), "
               "('O=C1O[C@@](OC2=C(C3=C(CO[C@H]([C@@H]3C)C)C(=C2)O)C)(CC=4C1=C(O)C(=C(O)C4C)C)C', "
               "'Ring is not aromatic'), "
               "('O=C1O[C@H](CC=2C1=C(O)C=C(OC)C2O)C[C@@H]3O[C@H](CCC3)C', "
               "'Ring is not aromatic'), "
               "('O=C1O[C@@H](CC=2C1=C(O)C=CC2O)CCC[C@@H](OC(=O)C[C@](O)(CCO)C)C', "
               "'Ring is not aromatic'), ('O=C1OCC=2C1=C(C(O)=C(OC)C2O)C', "
               "'Ring is not aromatic'), "
               "('O=C1OC(CC(=O)OC(CC(=O)OC(CC=2C=C(O)C=C(C2C(OC(CC(OC(CC=3C1=C(O)C=C(O)C3)C)=O)C)=O)O)C)CO)C', "
               "'Ring is not aromatic'), "
               "('O=C1O[C@@H](CC=2C1=C(O)C=CC2O)CCC[C@@H](OC(=O)CCC(=O)O)C', "
               "'Ring is not aromatic'), "
               "('O=C1O[C@H](OC)C=2C1=C(O)C(OC)=C(O)C2C', 'Ring is not "
               "aromatic'), "
               "('O=C1O[C@](CC)(C)C=2C1=C(O)C(=C(OC(=O)C3=C(O)C=C(O)C=C3C)C2)C', "
               "'Ring is not aromatic'), "
               "('O=C1O[C@H]([C@@H](O)C)C=2C1=C(O)C(=C(O)C2)C', 'Ring is not "
               "aromatic'), "
               "('O(C)C1=C(OC)C=C(C=C1OC)C(OCC=2N=C(COC(=O)C3=CC(OC)=C(C(=C3)OC)OC)C=CC2)=O', "
               "'No phenolic OH groups found on the aromatic ring'), "
               "('O=C1OC(=C)[C@@H](C)C=2C1=C(O)C=C(O)C2C=O', 'Ring is not "
               "aromatic'), ('O=C1O[C@]([C@@H](C)C=2C1=C(O)C=C(O)C2C)(CO)C', "
               "'Ring is not aromatic'), "
               "('O=C1O[C@@H]([C@@H](O)CC)CC=2C1=C(O)C=C(O)C2', 'Ring is not "
               "aromatic'), "
               "('O=C1O[C@@H](CC2=C(C=CC=C2)C(CC=3C1=C(O)C=C(O)C3)=O)C', 'Ring "
               "is not aromatic'), "
               "('O=C1O[C@H]([C@@](O)(C)C=2C1=C(O)C=C(O)C2C)CO', 'Ring is not "
               "aromatic'), "
               "('CN1C2=C(C=C(C=C2)S(=O)(=O)NC3=CC(=CC(=C3)C(=O)O)C(=O)O)C(=O)N(C1=O)C', "
               "'No phenolic OH groups found on the aromatic ring'), "
               "('Br[C@H]1C2=C(C(=O)O[C@H]1C)C(O)=CC(=C2)O', 'Ring is not "
               "aromatic'), ('O=C1O[C@@H](CCCCCC=2C1=C(O)C=C(O)C2)C', 'Ring is "
               "not aromatic'), ('O=C1O[C@H](/C=C/CCC)CC=2C1=C(O)C(=C(O)C2)C', "
               "'Ring is not aromatic')]",
    'attempt': 2,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 40,
    'num_false_positives': 100,
    'num_true_negatives': 6032,
    'num_false_negatives': 2,
    'num_negatives': None,
    'precision': 0.2857142857142857,
    'recall': 0.9523809523809523,
    'f1': 0.4395604395604395,
    'accuracy': 0.9834791059280855}


"""
Classifies: CHEBI:132023 S-acyl-4'-phosphopantetheine(2-)
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_S_acyl_4__phosphopantetheine_2__(smiles: str):
    """
    Determines if a molecule is an S-acyl-4'-phosphopantetheine(2-).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an S-acyl-4'-phosphopantetheine(2-), False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for phosphate group with 2 negative charges
    phosphate_pattern = Chem.MolFromSmarts("[O-]P(=O)([O-])O")
    if not mol.HasSubstructMatch(phosphate_pattern):
        return False, "Missing phosphate group with 2 negative charges"

    # Check for pantetheine core structure
    pantetheine_pattern = Chem.MolFromSmarts("NC(=O)CCNC(=O)C(O)C(C)(C)COP([O-])(=O)[O-]")
    if not mol.HasSubstructMatch(pantetheine_pattern):
        return False, "Missing pantetheine core structure"

    # Check for S-acyl group
    s_acyl_pattern = Chem.MolFromSmarts("CSC(=O)C")
    if not mol.HasSubstructMatch(s_acyl_pattern):
        return False, "Missing S-acyl group"

    # Get length of acyl chain
    acyl_chain_pattern = Chem.MolFromSmarts("CSC(=O)CCCCC")
    matches = mol.GetSubstructMatches(acyl_chain_pattern)
    if matches:
        chain_atoms = len(matches[0])
        return True, f"S-acyl-4'-phosphopantetheine(2-) with {chain_atoms-5} carbon acyl chain"
    
    return True, "S-acyl-4'-phosphopantetheine(2-)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:132023',
                          'name': "S-acyl-4'-phosphopantetheine(2-)",
                          'definition': 'A phosphopantetheine anion obtained '
                                        'by deprotonation of the phosphate OH '
                                        'groups of any '
                                        "S-acyl-4'-phosphopantetheine; major "
                                        'species at pH 7.3.',
                          'parents': ['CHEBI:67051']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 1,
    'num_true_negatives': 183913,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.6666666666666666,
    'recall': 1.0,
    'f1': 0.8,
    'accuracy': 0.9999945627351617}


"""
Classifies: CHEBI:190295 inorganic calcium salt
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors

def is_inorganic_calcium_salt(smiles: str):
    """
    Determines if a molecule is an inorganic calcium salt (lacks C-H bonds).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an inorganic calcium salt, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if molecule contains calcium
    if 'Ca' not in set([atom.GetSymbol() for atom in mol.GetAtoms()]):
        return False, "Molecule does not contain calcium"

    # Check for presence of C-H bonds
    for bond in mol.GetBonds():
        atom1 = mol.GetAtomWithIdx(bond.GetBeginAtomIdx())
        atom2 = mol.GetAtomWithIdx(bond.GetEndAtomIdx())
        if atom1.GetSymbol() == 'C' and atom2.GetSymbol() == 'H' or atom2.GetSymbol() == 'C' and atom1.GetSymbol() == 'H':
            return False, "Molecule contains C-H bonds"

    return True, "Inorganic calcium salt"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:190295',
                          'name': 'inorganic calcium salt',
                          'definition': 'A calcium salt that lacks C-H bonds',
                          'parents': ['CHEBI:167164', 'CHEBI:24839']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 73,
    'num_true_negatives': 183850,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.013513513513513514,
    'recall': 1.0,
    'f1': 0.026666666666666665,
    'accuracy': 0.9996030969313412}


"""
Classifies: CHEBI:15469 alk-2-enoyl-CoA
"""
from rdkit import Chem
from rdkit.Chem import Descriptors

def is_alk_2_enoyl_CoA(smiles: str):
    """
    Determines if a molecule is an alk-2-enoyl-CoA, which is an unsaturated fatty acyl-CoA that results from the
    formal condensation of the thiol group of coenzyme A with the carboxy group of any alpha,beta-unsaturated fatty acid.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an alk-2-enoyl-CoA, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for presence of CoA substructure
    coa_pattern = Chem.MolFromSmarts('C(=O)SCCNC(=O)CCNC(=O)[C@H](O)[C@H](O)[C@H](O)COP(=O)(O)OP(=O)(O)OC[C@@H]1O[C@H]([C@H](O)[C@@H]1OP(=O)(O)O)n1cnc2c(N)ncnc12')
    if mol.HasSubstructMatch(coa_pattern) is False:
        return False, "Molecule does not contain the CoA substructure"

    # Check for presence of alpha,beta-unsaturated carbonyl group
    alkene_carbonyl_pattern = Chem.MolFromSmarts('C=CC(=O)')
    if mol.HasSubstructMatch(alkene_carbonyl_pattern) is False:
        return False, "Molecule does not contain an alpha,beta-unsaturated carbonyl group"

    # Check for presence of fatty acid chain
    fatty_acid_pattern = Chem.MolFromSmarts('CCCCCCC')
    if mol.HasSubstructMatch(fatty_acid_pattern) is False:
        return False, "Molecule does not contain a fatty acid chain"

    return True, "Molecule is an alk-2-enoyl-CoA"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:15469',
                          'name': 'alk-2-enoyl-CoA',
                          'definition': 'An unsaturated fatty acyl-CoA that '
                                        'results from the formal condensation '
                                        'of the thiol group of coenzyme A with '
                                        'the carboxy group of any '
                                        'alpha,beta-unsaturated fatty acid.',
                          'parents': ['CHEBI:19573', 'CHEBI:51006']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183925,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999945630307842}


"""
Classifies: CHEBI:33555 arenesulfonic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_arenesulfonic_acid(smiles: str):
    """
    Determines if a molecule is an arenesulfonic acid.

    Arenesulfonic acids are organic derivatives of sulfonic acid in which the sulfo group is linked directly to carbon of an aryl group.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an arenesulfonic acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Find sulfonic acid groups
    sulfonic_acid_groups = [atom.GetIdx() for atom in mol.GetAtoms() if atom.GetSymbol() == 'S' and sum(1 for neighbor in atom.GetNeighbors() if neighbor.GetSymbol() == 'O') == 3]

    if not sulfonic_acid_groups:
        return False, "No sulfonic acid group found"

    # Check if sulfonic acid group is attached to an aromatic ring
    for idx in sulfonic_acid_groups:
        atom = mol.GetAtomWithIdx(idx)
        for neighbor in atom.GetNeighbors():
            if neighbor.GetIsAromatic():
                return True, "Sulfonic acid group attached to an aromatic ring"

    return False, "Sulfonic acid group not attached to an aromatic ring"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33555',
                          'name': 'arenesulfonic acid',
                          'definition': 'Organic derivatives of sulfonic acid '
                                        'in which the sulfo group is linked '
                                        'directly to carbon of an aryl group.',
                          'parents': ['CHEBI:33551']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 14,
    'num_false_positives': 100,
    'num_true_negatives': 43945,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.12280701754385964,
    'recall': 1.0,
    'f1': 0.21875,
    'accuracy': 0.997730316166958}


"""
Classifies: CHEBI:25168 mannosylinositol phosphorylceramide
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_mannosylinositol_phosphorylceramide(smiles: str):
    """
    Determines if a molecule is a mannosylinositol phosphorylceramide.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a mannosylinositol phosphorylceramide, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check for required elements
    elements = set([atom.GetSymbol() for atom in mol.GetAtoms()])
    required_elements = {'C', 'H', 'O', 'N', 'P'}
    if not required_elements.issubset(elements):
        return False, "Missing required elements (C, H, O, N, P)"
    
    # Check for phosphate group
    phos_pattern = Chem.MolFromSmarts('[P](=O)([O-,OH])([O-,OH])[O]')
    if not mol.HasSubstructMatch(phos_pattern):
        return False, "Missing phosphate group"
        
    # Check for ceramide core
    ceramide_pattern = Chem.MolFromSmarts('[CH2][CH]([OH])[CH]([NH][C](=O))[CH2]')
    if not mol.HasSubstructMatch(ceramide_pattern):
        return False, "Missing ceramide core structure"
        
    # Check for mannose
    mannose_pattern = Chem.MolFromSmarts('O[CH]1[CH](O)[CH](O)[CH](O)[CH](O)[CH]1O')
    if not mol.HasSubstructMatch(mannose_pattern):
        return False, "Missing mannose sugar"
        
    # Check for inositol
    inositol_pattern = Chem.MolFromSmarts('O[CH]1[CH](O)[CH](O)[CH](O)[CH](O)[CH]1O')
    if len(mol.GetSubstructMatches(inositol_pattern)) < 2: # Need at least 2 matches for both mannose and inositol
        return False, "Missing inositol group"
        
    # Check for long alkyl chains (ceramide tails)
    alkyl_pattern = Chem.MolFromSmarts('CCCCCCCC') # At least 8 carbons
    if len(mol.GetSubstructMatches(alkyl_pattern)) < 2:
        return False, "Missing required long alkyl chains"
        
    return True, "Contains mannosylinositol phosphorylceramide core structure with required functional groups"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25168',
                          'name': 'mannosylinositol phosphorylceramide',
                          'definition': 'A class of complex '
                                        'phosphoglycosphingolipids with a '
                                        'mannose-inositol-P head group. As '
                                        'with other ceramide derivatives, '
                                        'substituents R(1) and R(2) vary with '
                                        'different sphingoid bases and fatty '
                                        'acyl moieties.',
                          'parents': ['CHEBI:26057', 'CHEBI:60245']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183908,
    'num_false_negatives': 2,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999891251155456}


"""
Classifies: CHEBI:197491 fatty alcohol 21:0
"""
from rdkit import Chem

def is_fatty_alcohol_21_0(smiles: str):
    """
    Determines if a molecule is a fatty alcohol with 21 carbon atoms.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a fatty alcohol 21:0, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Count the number of carbon atoms
    num_carbons = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')

    # Check if there is exactly one oxygen atom
    num_oxygens = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'O')
    if num_oxygens != 1:
        return False, "Molecule does not contain exactly one oxygen atom"

    # Check if the molecule is aliphatic (no rings)
    if not mol.GetRingInfo().IsAtomInRingOfSize(0, 3) and \
       not mol.GetRingInfo().IsAtomInRingOfSize(0, 4) and \
       not mol.GetRingInfo().IsAtomInRingOfSize(0, 5) and \
       not mol.GetRingInfo().IsAtomInRingOfSize(0, 6) and \
       not mol.GetRingInfo().IsAtomInRingOfSize(0, 7):
        if num_carbons == 21:
            return True, "Molecule is a fatty alcohol 21:0"
        else:
            return False, f"Molecule has {num_carbons} carbon atoms (should be 21)"
    else:
        return False, "Molecule contains a ring"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:197491',
                          'name': 'fatty alcohol 21:0',
                          'definition': 'Any fatty alcohol containing 21 '
                                        'carbons.',
                          'parents': ['CHEBI:17135']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 100,
    'num_true_negatives': 60787,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.009900990099009901,
    'recall': 1.0,
    'f1': 0.0196078431372549,
    'accuracy': 0.998357640257522}


"""
Classifies: CHEBI:133075 class I yanuthone
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors

def is_class_I_yanuthone(smiles: str):
    """
    Determines if a molecule is a Class I yanuthone, defined as:
    Any of the yanuthones whose 5,6-epoxycyclohex-2-en-1-one core is substituted at position 3 by
    a methyl group or its oxidised (hydroxymethyl or acyloxymethyl) derivatives. It is thought that
    the core may be derived from 6-methylsalicylic acid, resulting in a C7 scaffold through a
    decarboxylation.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a Class I yanuthone, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of the 5,6-epoxycyclohex-2-en-1-one core
    core_smarts = '[C@]1(O)C=CC(=O)C2=C1C2(C)C'
    core_matches = mol.GetSubstructMatches(Chem.MolFromSmarts(core_smarts))

    if not core_matches:
        return False, "The 5,6-epoxycyclohex-2-en-1-one core is not present"

    # Check for substitution at position 3
    substituent_atoms = []
    for match in core_matches:
        atom_idx = match[-1]  # Index of the atom at position 3 (the methyl-substituted carbon)
        atom = mol.GetAtomWithIdx(atom_idx)
        substituent_atoms.extend([neighbor.GetSymbol() for neighbor in atom.GetNeighbors() if neighbor.GetIdx() not in match])

    if 'C' not in substituent_atoms:
        return False, "The core is not substituted at position 3 by a methyl group or its oxidised derivatives"

    # Check for the presence of hydroxymethyl or acyloxymethyl substituents
    oxidised_substituents = ['O', 'C(=O)O']
    oxidised_substituents_present = any(s in substituent_atoms for s in oxidised_substituents)

    if oxidised_substituents_present:
        return True, "Class I yanuthone with oxidised substituent at position 3"
    else:
        return True, "Class I yanuthone with methyl substituent at position 3"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:133075',
                          'name': 'class I yanuthone',
                          'definition': 'Any of the yanuthones whose '
                                        '5,6-epoxycyclohex-2-en-1-one core is '
                                        'substituted at position 3 by a methyl '
                                        'group or its oxidised (hydroxymethyl '
                                        'or acyloxymethyl) derivatives. It is '
                                        'thought that the core may be derived '
                                        'from 6-methylsalicylic acid, '
                                        'resulting in a C7 scaffold through a '
                                        'decarboxylation.',
                          'parents': ['CHEBI:133070']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183923,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999945629716622}


"""
Classifies: CHEBI:15722 peptidyl amide
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_peptidyl_amide(smiles: str) -> tuple[bool, str]:
    """
    Determines if a molecule is a peptidyl amide (a peptide with a carbamoyl group at the C-terminus).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is a peptidyl amide, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for peptide bonds
    peptide_bonds = AllChem.MMFFGetMoleculeProperties(mol).GetPropInt('NumPeptideBonds', 0)
    if peptide_bonds < 1:
        return False, "Molecule does not contain any peptide bonds"

    # Check for carbamoyl group at C-terminus
    carbamoyl_found = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C' and atom.GetDegree() == 3:
            neighbors = [mol.GetAtomWithIdx(n).GetSymbol() for n in atom.GetNeighbors()]
            if 'O' in neighbors and 'N' in neighbors and 'N' in neighbors:
                carbamoyl_found = True
                break

    if carbamoyl_found:
        return True, "Molecule is a peptidyl amide (peptide with carbamoyl group at C-terminus)"
    else:
        return False, "Molecule does not have a carbamoyl group at the C-terminus"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:15722',
                          'name': 'peptidyl amide',
                          'definition': 'A peptide that has a carbamoyl group '
                                        'at the C-terminus.',
                          'parents': ['CHEBI:16670']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "'MMFFMolProperties' object has no attribute 'GetPropInt'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:23990 ethyl ester
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_ethyl_ester(smiles: str):
    """
    Determines if a molecule contains an ethyl ester group.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule contains ethyl ester, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # SMARTS pattern for ethyl ester: C-C-O-C(=O)-
    ethyl_ester_pattern = Chem.MolFromSmarts('CCO[C;H0](=O)')
    
    if mol.HasSubstructMatch(ethyl_ester_pattern):
        # Find all matches
        matches = mol.GetSubstructMatches(ethyl_ester_pattern)
        
        # Count number of ethyl ester groups
        num_matches = len(matches)
        
        if num_matches == 1:
            return True, "Contains 1 ethyl ester group"
        else:
            return True, f"Contains {num_matches} ethyl ester groups"
            
    return False, "No ethyl ester group found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23990',
                          'name': 'ethyl ester',
                          'definition': 'Any carboxylic ester resulting from '
                                        'the formal condensation of the '
                                        'carboxy group of a carboxylic acid '
                                        'with ethanol.',
                          'parents': ['CHEBI:33308']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 20,
    'num_false_positives': 100,
    'num_true_negatives': 591,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.16666666666666666,
    'recall': 1.0,
    'f1': 0.2857142857142857,
    'accuracy': 0.8593530239099859}


"""
Classifies: CHEBI:24068 fluoroamino acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_fluoroamino_acid(smiles: str):
    """
    Determines if a molecule is a fluoroamino acid (an organofluorine compound consisting
    of an amino acid substituted by a fluoro group).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a fluoroamino acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of a carboxylic acid group
    carboxylic_acid_present = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O' and atom.GetDegree() == 1:
            neighbor = atom.GetNeighbors()[0]
            if neighbor.GetSymbol() == 'C' and neighbor.GetDegree() == 3:
                carboxylic_acid_present = True
                break

    if not carboxylic_acid_present:
        return False, "No carboxylic acid group found"

    # Check for the presence of an amino group
    amino_group_present = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'N' and atom.GetDegree() == 3:
            amino_group_present = True
            break

    if not amino_group_present:
        return False, "No amino group found"

    # Check for the presence of at least one fluorine atom
    fluorine_present = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'F':
            fluorine_present = True
            break

    if not fluorine_present:
        return False, "No fluorine atom found"

    return True, "The molecule is a fluoroamino acid"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24068',
                          'name': 'fluoroamino acid',
                          'definition': 'An organofluorine compound that '
                                        'consists of an amino acid substituted '
                                        'by a fluoro group.',
                          'parents': ['CHEBI:24470', 'CHEBI:37143']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 100,
    'num_true_negatives': 3139,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0,
    'accuracy': 0.9688271604938271}


"""
Classifies: CHEBI:33563 glycolipid
"""
from rdkit import Chem

def is_glycolipid(smiles: str):
    """
    Determines if a molecule is a glycolipid based on its SMILES string.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a glycolipid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of a carbohydrate part
    carbohydrate_part = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O' and atom.GetTotalDegree() == 2:
            neighbors = [mol.GetAtomWithIdx(neighbor.GetIdx()).GetSymbol() for neighbor in atom.GetNeighbors()]
            if 'C' in neighbors and 'C' in neighbors:
                carbohydrate_part = True
                break

    if not carbohydrate_part:
        return False, "No carbohydrate part found"

    # Check for the presence of a lipid part
    lipid_part = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C' and atom.GetTotalDegree() == 4:
            neighbors = [mol.GetAtomWithIdx(neighbor.GetIdx()).GetSymbol() for neighbor in atom.GetNeighbors()]
            if 'C' in neighbors and 'C' in neighbors and 'C' in neighbors and ('O' in neighbors or 'N' in neighbors):
                lipid_part = True
                break

    if not lipid_part:
        return False, "No lipid part found"

    return True, "Molecule contains both carbohydrate and lipid parts"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33563',
                          'name': 'glycolipid',
                          'definition': 'Any member of class of '
                                        '1,2-di-O-acylglycerols joined at '
                                        'oxygen 3 by a glycosidic linkage to a '
                                        'carbohydrate part (usually a mono-, '
                                        'di- or tri-saccharide). Some '
                                        'substances classified as bacterial '
                                        'glycolipids have the sugar part '
                                        'acylated by one or more fatty acids '
                                        'and the glycerol part may be absent.',
                          'parents': ['CHEBI:35740']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': 'Attempt failed: Python argument types in\n'
               '    Mol.GetAtomWithIdx(Mol, Atom)\n'
               'did not match C++ signature:\n'
               '    GetAtomWithIdx(RDKit::ROMol {lvalue} self, unsigned int '
               'idx)',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 161,
    'num_false_positives': 100,
    'num_true_negatives': 42,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.6168582375478927,
    'recall': 1.0,
    'f1': 0.7630331753554502,
    'accuracy': 0.66996699669967}


"""
Classifies: CHEBI:23075 glycotetraosylceramide
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.MolStandardize import rdMolStandardize

def is_glycotetraosylceramide(smiles: str):
    """
    Determines if a molecule is a glycotetraosylceramide.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a glycotetraosylceramide, False otherwise
        str: Reason for classification
    """
    # Check for valid SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Standardize the molecule
    uncharger = rdMolStandardize.Uncharger()
    mol = uncharger.uncharge(mol)
    
    # Count rings that could be sugars (5-6 membered rings with multiple OH groups)
    rings = mol.GetRingInfo()
    sugar_ring_count = 0
    
    for ring in rings.AtomRings():
        if len(ring) in [5,6]:
            ring_atoms = [mol.GetAtomWithIdx(i) for i in ring]
            # Count oxygen atoms in ring
            ring_o_count = sum(1 for atom in ring_atoms if atom.GetSymbol() == 'O')
            # Count OH groups attached to ring
            oh_count = 0
            for atom_idx in ring:
                atom = mol.GetAtomWithIdx(atom_idx)
                for neighbor in atom.GetNeighbors():
                    if neighbor.GetSymbol() == 'O' and neighbor.GetTotalNumHs() == 1:
                        oh_count += 1
            if ring_o_count >= 1 and oh_count >= 2:
                sugar_ring_count += 1
                
    if sugar_ring_count != 4:
        return False, f"Found {sugar_ring_count} sugar rings, expected 4"

    # Check for ceramide moiety
    # Look for N-C(=O) pattern
    amide_pattern = Chem.MolFromSmarts('[N;H1]-C(=O)')
    if not mol.HasSubstructMatch(amide_pattern):
        return False, "No amide group found (required for ceramide)"
        
    # Look for long alkyl chains
    alkyl_pattern = Chem.MolFromSmarts('CCCCCCCCC')  # At least 9 carbons
    if len(mol.GetSubstructMatches(alkyl_pattern)) < 2:
        return False, "Missing required long alkyl chains"

    # Check for glycosidic linkages between sugars
    glycosidic_pattern = Chem.MolFromSmarts('[C]-O-[C]')
    glycosidic_bonds = len(mol.GetSubstructMatches(glycosidic_pattern))
    
    if glycosidic_bonds < 4:
        return False, f"Found {glycosidic_bonds} glycosidic bonds, expected at least 4"
        
    return True, "Molecule contains 4 sugar rings, ceramide moiety, and appropriate glycosidic linkages"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23075',
                          'name': 'glycotetraosylceramide',
                          'definition': 'An oligoglycosylceramide consisting '
                                        'of a glycotetraosyl moiety attached '
                                        'to the ceramide oxygen with an '
                                        'unspecified N-acyl substituent '
                                        'attached to the ceramide nitrogen.',
                          'parents': ['CHEBI:36520']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 100,
    'num_true_negatives': 53634,
    'num_false_negatives': 2,
    'num_negatives': None,
    'precision': 0.02912621359223301,
    'recall': 0.6,
    'f1': 0.05555555555555556,
    'accuracy': 0.9981019371406241}


"""
Classifies: CHEBI:27933 beta-lactam antibiotic
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_beta_lactam_antibiotic(smiles: str):
    """
    Determines if a molecule is a beta-lactam antibiotic.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a beta-lactam antibiotic, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for beta-lactam ring
    beta_lactam_ring = False
    for ring in mol.GetRingInfo().AtomRings():
        if len(ring) == 4:
            atoms = [mol.GetAtomWithIdx(idx) for idx in ring]
            if any(atom.GetSymbol() == 'N' for atom in atoms) and any(atom.GetSymbol() == 'O' for atom in atoms):
                beta_lactam_ring = True
                break

    if not beta_lactam_ring:
        return False, "No beta-lactam ring found"

    # Check for nitrogen in the molecule
    if not any(atom.GetSymbol() == 'N' for atom in mol.GetAtoms()):
        return False, "No nitrogen atoms found"

    # Check if the molecule is an antibiotic
    # (This is a simplified check based on the presence of specific functional groups)
    antibiotic_groups = ['C(=O)N', 'C(=O)O', 'S=O', 'N=O', 'C#N']
    has_antibiotic_group = False
    for group in antibiotic_groups:
        if mol.HasSubstructMatch(Chem.MolFromSmarts(group)):
            has_antibiotic_group = True
            break

    if not has_antibiotic_group:
        return False, "No antibiotic functional groups found"

    return True, "Molecule is a beta-lactam antibiotic"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:27933',
                          'name': 'beta-lactam antibiotic',
                          'definition': 'An organonitrogen heterocyclic '
                                        'antibiotic that contains a '
                                        'beta-lactam ring.',
                          'parents': ['CHEBI:25558', 'CHEBI:35627']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183607,
    'num_false_negatives': 32,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.999825745075937}


"""
Classifies: CHEBI:16826 acyl monophosphate
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_acyl_monophosphate(smiles: str):
    """
    Determines if a molecule is an acyl monophosphate, defined as an organic phosphate
    ester or anhydride formed by condensation of phosphoric acid with a carboxylic acid.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an acyl monophosphate, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of a phosphate group
    phosphate_atoms = [atom.GetIdx() for atom in mol.GetAtoms() if atom.GetSymbol() == 'P']
    if not phosphate_atoms:
        return False, "No phosphorus atom found"

    # Check for the presence of a carbonyl group
    carbonyl_atoms = [atom.GetIdx() for atom in mol.GetAtoms() if atom.GetSymbol() == 'C' and sum(mol.GetAtomWithIdx(atom).GetTotalNumHs() for atom in mol.GetAtomWithIdx(atom).GetNeighbors()) == 0]
    if not carbonyl_atoms:
        return False, "No carbonyl group found"

    # Check if the phosphate group is connected to the carbonyl group
    for p_idx in phosphate_atoms:
        p_atom = mol.GetAtomWithIdx(p_idx)
        for neighbor in p_atom.GetNeighbors():
            if neighbor.GetIdx() in carbonyl_atoms:
                return True, "Molecule is an acyl monophosphate"

    return False, "Molecule does not meet the criteria for an acyl monophosphate"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:16826',
                          'name': 'acyl monophosphate',
                          'definition': 'An organic phosphate ester or '
                                        'anhydride formed by condensation of '
                                        'phosphoric acid with a carboxylic '
                                        'acid.',
                          'parents': ['CHEBI:25703', 'CHEBI:37787']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': 'Python argument types in\n'
             '    Mol.GetAtomWithIdx(Mol, Atom)\n'
             'did not match C++ signature:\n'
             '    GetAtomWithIdx(RDKit::ROMol {lvalue} self, unsigned int idx)',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:22951 butanone
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_butanone(smiles: str):
    """
    Determines if a molecule is a butanone (ketone with a butane chain and oxo group).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a butanone, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Look for ketone groups (C=O)
    pattern = Chem.MolFromSmarts('[C;!$(C=O)]-C(=O)-[C;!$(C=O)]')
    matches = mol.GetSubstructMatches(pattern)
    
    if not matches:
        return False, "No ketone group found"
        
    for match in matches:
        # Get the carbon atoms connected to the carbonyl carbon
        carbonyl_carbon = mol.GetAtomWithIdx(match[1])
        neighbors = [n.GetIdx() for n in carbonyl_carbon.GetNeighbors() if n.GetAtomicNum() == 6]
        
        for neighbor in neighbors:
            # Try to find a chain of 4 carbons including the carbonyl carbon
            visited = set([match[1]])  # Start with carbonyl carbon
            current = neighbor
            chain_length = 1
            
            while chain_length < 4:
                visited.add(current)
                next_carbons = [n.GetIdx() for n in mol.GetAtomWithIdx(current).GetNeighbors() 
                              if n.GetAtomicNum() == 6 and n.GetIdx() not in visited]
                
                if not next_carbons:
                    break
                    
                current = next_carbons[0]
                chain_length += 1
                
            if chain_length == 4:
                return True, "Found butanone structure: 4-carbon chain with ketone group"
                
    return False, "No butanone structure found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22951',
                          'name': 'butanone',
                          'definition': 'Any  ketone that is butane '
                                        'substituted by an oxo group at '
                                        'unspecified position.',
                          'parents': ['CHEBI:17087']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 100,
    'num_true_negatives': 1017,
    'num_false_negatives': 3,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9080357142857143}


"""
Classifies: CHEBI:25529 pyridinecarboxamide
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_pyridinecarboxamide(smiles: str):
    """
    Determines if a molecule is a pyridinecarboxamide (pyridine with at least one carboxamide substituent).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a pyridinecarboxamide, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
    
    # Check for pyridine ring
    pyridine_pattern = Chem.MolFromSmarts('n1ccccc1')
    if not mol.HasSubstructMatch(pyridine_pattern):
        return False, "No pyridine ring found"
        
    # Check for carboxamide group attached to pyridine
    # Pattern matches pyridine ring with carboxamide substituent
    pyridine_carboxamide_pattern = Chem.MolFromSmarts('n1ccccc1C(=O)N')
    
    # Also check for N-substituted carboxamide
    n_subst_carboxamide_pattern = Chem.MolFromSmarts('n1ccccc1C(=O)N[#6,#1]')
    
    if not (mol.HasSubstructMatch(pyridine_carboxamide_pattern) or 
            mol.HasSubstructMatch(n_subst_carboxamide_pattern)):
        return False, "No carboxamide substituent found on pyridine ring"
    
    # Get number of matches to report in reason
    matches = len(mol.GetSubstructMatches(pyridine_carboxamide_pattern))
    matches += len(mol.GetSubstructMatches(n_subst_carboxamide_pattern))
    
    return True, f"Pyridine ring found with {matches} carboxamide substituent(s)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25529',
                          'name': 'pyridinecarboxamide',
                          'definition': 'A member of the class of pyridines '
                                        'that is a substituted pyridine in '
                                        'which at least one of the '
                                        'substituents is a carboxamide or '
                                        'N-substituted caraboxamide group.',
                          'parents': ['CHEBI:26421', 'CHEBI:37622']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 6,
    'num_false_positives': 100,
    'num_true_negatives': 30014,
    'num_false_negatives': 26,
    'num_negatives': None,
    'precision': 0.05660377358490566,
    'recall': 0.1875,
    'f1': 0.08695652173913045,
    'accuracy': 0.9958203410070988}


"""
Classifies: CHEBI:131874 HEPE(1-)
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_HEPE_1__(smiles: str):
    """
    Determines if a molecule is a HEPE(1-) (an unsaturated fatty acid anion arising from deprotonation of the carboxylic acid function of any hydroxyicosapentaenoic acid).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a HEPE(1-), False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for 20 heavy atoms (carbon, oxygen, and hydrogen)
    if Descriptors.HeavyAtomCount(mol) != 20:
        return False, "Incorrect number of heavy atoms"

    # Check for 5 double bonds
    if Descriptors.NumHeteroatomicDoubleBonds(mol) != 5:
        return False, "Incorrect number of double bonds"

    # Check for 1 oxygen atom
    if Descriptors.NHOHCount(mol) != 1:
        return False, "Incorrect number of oxygen atoms"

    # Check for 1 carboxylate anion group
    carboxylate_found = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O' and atom.GetFormalCharge() == -1:
            neighbors = [mol.GetAtomWithIdx(n).GetSymbol() for n in atom.GetNeighbors()]
            if 'C' in neighbors and 'O' in neighbors:
                carboxylate_found = True
                break

    if not carboxylate_found:
        return False, "No carboxylate anion group found"

    # Check for 1 hydroxyl group
    hydroxyl_found = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O' and atom.GetFormalCharge() == 0:
            neighbors = [mol.GetAtomWithIdx(n).GetSymbol() for n in atom.GetNeighbors()]
            if 'C' in neighbors and 'H' in neighbors:
                hydroxyl_found = True
                break

    if not hydroxyl_found:
        return False, "No hydroxyl group found"

    return True, "Molecule is a HEPE(1-)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:131874',
                          'name': 'HEPE(1-)',
                          'definition': 'An unsaturated fatty acid anion '
                                        'arising from deprotonation of the '
                                        'carboxylic acid function of any '
                                        'hydroxyicosapentaenoic acid (HEPE).',
                          'parents': ['CHEBI:131871', 'CHEBI:62937']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "module 'rdkit.Chem.Descriptors' has no attribute "
             "'NumHeteroatomicDoubleBonds'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:26596 salicylates
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_salicylates(smiles: str):
    """
    Determines if a molecule is a salicylate (salt or ester of salicylic acid).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a salicylate, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Pattern for salicylic acid core structure (benzene with ortho COOH and OH)
    salicylic_pattern = Chem.MolFromSmarts('[cH]1[cH][cH][cH][cH]c1[CX3](=O)[OX2,OX1-]')
    
    # Pattern for ester group
    ester_pattern = Chem.MolFromSmarts('[#6][CX3](=O)[OX2][#6,#83]') # #83 is Bi
    
    # Pattern for salt form (metal-oxygen bond)
    salt_pattern = Chem.MolFromSmarts('[OX2,OX1-]-[#3,#11,#12,#13,#19,#20,#21,#22,#23,#24,#25,#26,#27,#28,#29,#30,#31,#32,#33,#34,#35,#36,#37,#38,#39,#40,#41,#42,#83]')

    if not mol.HasSubstructMatch(salicylic_pattern):
        return False, "No salicylic acid core structure found"

    # Check for ester form
    if mol.HasSubstructMatch(ester_pattern):
        # Check if the ester is formed at carboxyl or phenol position
        matches = mol.GetSubstructMatches(ester_pattern)
        for match in matches:
            if any(idx in match for idx in mol.GetSubstructMatch(salicylic_pattern)):
                return True, "Salicylate ester found"

    # Check for salt form
    if mol.HasSubstructMatch(salt_pattern):
        matches = mol.GetSubstructMatches(salt_pattern)
        for match in matches:
            if any(idx in match for idx in mol.GetSubstructMatch(salicylic_pattern)):
                metal_idx = match[1]
                metal_atom = mol.GetAtomWithIdx(metal_idx)
                return True, f"Salicylate salt with {metal_atom.GetSymbol()} found"
                
    # Check for the special case of bismuth subsalicylate
    if 'Bi' in smiles and 'O[Bi]' in smiles:
        return True, "Bismuth subsalicylate found"

    return False, "Not a salicylate derivative"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26596',
                          'name': 'salicylates',
                          'definition': 'Any salt or ester arising from '
                                        'reaction of the carboxy group of '
                                        'salicylic acid, or any ester '
                                        'resulting from the condensation of '
                                        'the phenolic hydroxy group of '
                                        'salicylic acid with an organic acid.',
                          'parents': ['CHEBI:36963']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 100,
    'num_true_negatives': 42063,
    'num_false_negatives': 2,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9975809320526503}


"""
Classifies: CHEBI:25392 naphthols
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_naphthols(smiles: str):
    """
    Determines if a molecule is a naphthol (hydroxynaphthalene derivative with single OH group).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a naphthol, False otherwise
        str: Reason for classification
    """
    # Parse SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check for naphthalene core
    rings = mol.GetRingInfo()
    
    # Find fused 6-membered rings
    fused_rings = []
    for ring1 in rings.AtomRings():
        if len(ring1) != 6:
            continue
        for ring2 in rings.AtomRings():
            if len(ring2) != 6 or ring1 == ring2:
                continue
            # Check if rings share exactly 2 atoms
            shared = set(ring1).intersection(set(ring2))
            if len(shared) == 2:
                fused_rings.append((ring1, ring2))
                
    if not fused_rings:
        return False, "No fused 6-membered rings found"
        
    # Check for naphthalene core (two fused aromatic 6-membered rings)
    naphthalene_found = False
    for ring1, ring2 in fused_rings:
        atoms1 = [mol.GetAtomWithIdx(i) for i in ring1]
        atoms2 = [mol.GetAtomWithIdx(i) for i in ring2]
        
        if (all(atom.GetIsAromatic() for atom in atoms1) and 
            all(atom.GetIsAromatic() for atom in atoms2) and
            all(atom.GetSymbol() == 'C' for atom in atoms1) and 
            all(atom.GetSymbol() == 'C' for atom in atoms2)):
            naphthalene_found = True
            naphthalene_atoms = set(ring1).union(set(ring2))
            break
            
    if not naphthalene_found:
        return False, "No naphthalene core found"
        
    # Count hydroxy groups
    hydroxy_count = 0
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O':
            # Check if oxygen is part of a hydroxy group
            neighbors = [n for n in atom.GetNeighbors()]
            if len(neighbors) == 1 and neighbors[0].GetSymbol() == 'C':
                hydroxy_count += 1
                
    if hydroxy_count == 0:
        return False, "No hydroxy groups found"
    elif hydroxy_count > 1:
        return False, f"Multiple ({hydroxy_count}) hydroxy groups found"
        
    # Check if at least one OH is attached to naphthalene core
    oh_on_core = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O':
            neighbors = [n for n in atom.GetNeighbors()]
            if (len(neighbors) == 1 and 
                neighbors[0].GetSymbol() == 'C' and 
                neighbors[0].GetIdx() in naphthalene_atoms):
                oh_on_core = True
                break
                
    if not oh_on_core:
        return False, "Hydroxy group not attached to naphthalene core"
        
    return True, "Single hydroxy group attached to naphthalene core"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25392',
                          'name': 'naphthols',
                          'definition': 'Any hydroxynaphthalene derivative '
                                        'that has a single hydroxy '
                                        'substituent.',
                          'parents': ['CHEBI:24727']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 96,
    'num_true_negatives': 183643,
    'num_false_negatives': 16,
    'num_negatives': None,
    'precision': 0.030303030303030304,
    'recall': 0.15789473684210525,
    'f1': 0.05084745762711865,
    'accuracy': 0.999390502726412}


"""
Classifies: CHEBI:144368 1,2-diacyl-sn-glycerolipid
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_1_2_diacyl_sn_glycerolipid(smiles: str):
    """
    Determines if a molecule is a 1,2-diacyl-sn-glycerolipid.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a 1,2-diacyl-sn-glycerolipid, False otherwise
        str: Reason for classification
    """
    # Check for valid SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Look for glycerol backbone pattern with correct stereochemistry
    glycerol_pattern = Chem.MolFromSmarts("[OX2][CH2X4][C@@H]([OX2])[CH2X4][OX2]")
    if not mol.HasSubstructMatch(glycerol_pattern):
        glycerol_pattern_alt = Chem.MolFromSmarts("[OX2][CH2X4][C@H]([OX2])[CH2X4][OX2]")
        if not mol.HasSubstructMatch(glycerol_pattern_alt):
            return False, "Missing glycerol backbone with correct stereochemistry"

    # Look for acyl groups at positions 1,2
    acyl_pattern = Chem.MolFromSmarts("[CX3](=O)[OX2]")
    acyl_matches = mol.GetSubstructMatches(acyl_pattern)
    if len(acyl_matches) < 1:
        return False, "Missing required acyl group(s)"

    # Check R3 position for allowed groups
    # Updated patterns to be more specific
    r3_h_pattern = Chem.MolFromSmarts("[CH2X4][OH]")
    r3_phosphate_pattern = Chem.MolFromSmarts("[CH2X4]OP(=O)([O-])[O-]")
    r3_phospholipid_pattern = Chem.MolFromSmarts("[CH2X4]OP(=O)([O-])O[CH2,CH3]")

    if mol.HasSubstructMatch(r3_phosphate_pattern):
        return True, "Valid 1,2-diacyl-sn-glycerolipid with R3=phosphate"
    elif mol.HasSubstructMatch(r3_phospholipid_pattern):
        return True, "Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid"
    elif mol.HasSubstructMatch(r3_h_pattern):
        return True, "Valid 1,2-diacyl-sn-glycerolipid with R3=H"
    else:
        # Check for wildcard patterns that might indicate R3 groups
        r3_wildcard_pattern = Chem.MolFromSmarts("[CH2X4][OX2][*]")
        if mol.HasSubstructMatch(r3_wildcard_pattern):
            return True, "Valid 1,2-diacyl-sn-glycerolipid with R3=H"

    return False, "R3 position does not match allowed groups (H, phosphate, or phospholipid)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:144368',
                          'name': '1,2-diacyl-sn-glycerolipid',
                          'definition': 'A diradylglycerolipid where R1 and R2 '
                                        'are acyl chains and R3 can be an H, a '
                                        'phosphate or a phospholipid group.',
                          'parents': ['CHEBI:35741', 'CHEBI:50860']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.019417475728155338 is too low.\n'
               "True positives: [('OC[C@@H](CO\\\\C=C/[*])OC([*])=O', 'Valid "
               "1,2-diacyl-sn-glycerolipid with R3=H')]\n"
               'False positives: '
               "[('C(C(COC(CCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)=O)O/C=C\\\\CCCCCCCC/C=C\\\\CCCCCC)OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('[C@](COC(CCCCCCCCCCCCC)=O)(OC(=O)CCCCCCCCCCCCC)([H])COP(OCCC[S+](C)C)([O-])=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('P(OCC[N+](C)(C)C)(OC[C@H](OC(=O)C(C(C(C(C(C(C(C(C(C(C(C(C([2H])([2H])[2H])([2H])[2H])([2H])[2H])([2H])[2H])([2H])[2H])([2H])[2H])([2H])[2H])([2H])[2H])([2H])[2H])([2H])[2H])([2H])[2H])([2H])[2H])([2H])[2H])COC(=O)C(C(C(C(C(C(C(C(C(C(C(C(C([2H])([2H])[2H])([2H])[2H])([2H])[2H])([2H])[2H])([2H])[2H])([2H])[2H])([2H])[2H])([2H])[2H])([2H])[2H])([2H])[2H])([2H])[2H])([2H])[2H])([2H])[2H])([O-])=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('O1[C@@H]([C@H](O)C(O)C(O)[C@@H]1OC[C@H](OC(=O)CCCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)COC(=O)C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)CO[C@H]2O[C@@H]([C@H](O)C(O)C2O)CO', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('P(OCC[N+](C)(C)C)(OC[C@H](OC(=O)CCCCCCC/C=C\\\\CCCCCC)COC(=O)CCCCCCC/C=C\\\\CCCCCCC)([O-])=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('S([C@@H]([C@@H](O)CCCC(O)=O)/C=C/C=C/C=C\\\\C/C=C\\\\CCCCC)C[C@H](N)C(O[C@H](COC(=O)CCCCCCCCC)CO)=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('CCCCCCCCCCCCCCCCCC(=O)O[C@H](COC(=O)CCCCCCC\\\\C=C/CCCCCCCC)COP([O-])(=O)OC[C@H]([NH3+])C([O-])=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('C([C@@](COC(CCCCCCC/C=C\\\\CCCC)=O)(OC(CC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)=O)[H])OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('P(OC[C@H](OC(=O)CCC(O)/C=C/C(O)=O)COC(=O)CCCCCCCCCCCCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('N[C@@H](COP(O)(=O)OC[C@H](O)CO)C(O)=O', 'Valid "
               "1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('P(OCC[N+](C)(C)C)(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCCCC)COC(=O)CCCCCCCCCCC)([O-])=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('CCCCCCCCCCCC(=O)OC[C@H](COP([O-])([O-])=O)OC(=O)CCCCCCCCCCC', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phosphate'), "
               "('CCCCCCCCCCCCCCCC(=O)OC[C@H](COP([O-])(=O)OCC[N+](C)(C)C)OC(=O)CCCC([O-])=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('P(OCC[N+](C)(C)C)(OC[C@H](OC(=O)C=C)COCCCCCCCCCCCCCCCC)([O-])=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('P(OC[C@H](OC(=O)CCCCCCC/C=C\\\\CCCCCCCC)CO/C=C\\\\CCCCCCCCCCCCCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('O=C(CCC/C=C\\\\C\\\\C=C/C=C/C(C/C=C\\\\CCCCC)OO)OC(CO)CO', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('P(OC[C@H](OC(=O)CCCCCCC/C=C\\\\CCCCCCC)COC(=O)CCCCCCC/C=C\\\\CCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('[C@](CO/C=C\\\\CCCCCCCCCCCCCC)(OC(=O)CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)([H])COP(OCC[N+](C)(C)C)([O-])=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('O(C[C@](CO)(OC(CCCCCCCCC/C=C\\\\CCCCCC)=O)[H])C(CC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('C(C(COC(CC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)=O)O/C=C\\\\CCCCCCCCCCCCCC)OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC)COC(=O)CCCCCCC/C=C\\\\CCCCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('CCCCCCCCCCCCCCCCCCOC[C@H](COP([O-])(=O)OCC[N+](C)(C)C)OC(=O)CCCCCCCCCCC\\\\C=C/C\\\\C=C/CCCCC', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('O(C(=O)CCCCCCCCCCCCCC)C[C@@H](OC(=O)CCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC)CO', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('[C@H]1(O)[C@@H](OC[C@H]2O[C@H]([C@@H]([C@H]([C@H]2O)O)O)OC[C@@H](COC(=O)*)OC(*)=O)O[C@H](CO)[C@@H]([C@@H]1O)O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('CCCCCCCCCCCCCCCCC(=O)OC[C@@H](O)COP([O-])([O-])=O', 'Valid "
               "1,2-diacyl-sn-glycerolipid with R3=phosphate'), "
               "('O[C@H]1[C@@H]([C@H](C(=O)C1)C/C=C\\\\CCCC(OC[C@@H](OC(=O)CCCCCCCCCCCCCCCC)CO)=O)/C=C/[C@@H](O)CCCCC', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('CCCCCCCCCCCCCCCC(=O)OC[C@H](COP([O-])(=O)OCC(O)CO)OC(=O)CCCCCCC\\\\C=C/CCCCCCCC', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('C([C@@](COC(CCCCCCCCCCCCC)=O)(OC(CCCCCCC/C=C\\\\CCCC)=O)[H])OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('C([C@H](NC(*)=O)C([O-])=O)OP(=O)([O-])OC[C@H](O)COC(=O)*', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('CCCCCCCCCCCCCC(=O)OCC(O)CO', 'Valid "
               "1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('C([C@@](COC(CCCCCCC/C=C\\\\C/C=C\\\\CCCCC)=O)(OC(CCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC)=O)[H])O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('C([C@@](COC(CCCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CC)=O)(OC(CCCCCCC/C=C\\\\CCCCCC)=O)[H])OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCC(=O)OC[C@H](CO)OC([*])=O', 'Valid "
               "1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('C([C@@](COC(CCCCCCCCCCC/C=C\\\\CCCCCCCC)=O)(OC(CCCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CC)=O)[H])O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('[H][C@@](COC(=O)CCCCCCC\\\\C=C/CCCCCCCC)(COP([O-])([O-])=O)OC(=O)CCCCCCC\\\\C=C/CCCCCC', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phosphate'), "
               "('C([C@@](COC(CCCCCCCCCCCCC/C=C\\\\CCCCCCCC)=O)(OC(CCCCCCC/C=C\\\\CCCC)=O)[H])O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('C(\\\\OC[C@H](COP([O-])(=O)OCC[NH3+])OC(=O)*)=C\\\\CCCCCCCCCCCCCCCC', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('C(OC[C@H](COP(OCC[N+](C)(C)C)(=O)[O-])O)(*)=O', 'Valid "
               "1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('C([C@@](COC(CCCCCCCCCCC/C=C\\\\CCCCCCCC)=O)(OC(CCCCCCCCC/C=C\\\\CCCCCCCC)=O)[H])OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('C([C@@](COC(CCCCCCC/C=C\\\\C/C=C\\\\CCCCC)=O)(OC(CCCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CC)=O)[H])OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('P([O-])(=O)(OC[C@@H](C(=O)[O-])[NH3+])OC[C@@H](COC(=O)*)OC(=O)*', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCC(=O)OCC(CO[C@@H]1O[C@H](CO)[C@H](O)[C@H](O)[C@H]1O)OC(=O)CCCCCCC\\\\C=C/CCCCCCCC', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('C([C@@](CO/C=C\\\\CCCCCCCCCCCCCC)(OC(CC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)=O)[H])OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('C([C@@](COC(CCCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CC)=O)(OC(CCCCCCCCCCCCCCCCC)=O)[H])OP(=O)(O)OC[C@@](CO)([H])O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('P(OCC[N+](C)(C)C)(OC[C@H](OC(=O)C)COCCCCCCCCCC/C=C\\\\CCCC)([O-])=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('C([C@@](COC(CCCCCCCCCCCCC)=O)(OC(CCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)=O)[H])O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('C([C@@](COC(CCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)=O)(OC(CCCCCCCCC/C=C\\\\CCCCCCCC)=O)[H])O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('CCCCCCCCCCCCCCCC(=O)OCC(O)COP([O-])([O-])=O', 'Valid "
               "1,2-diacyl-sn-glycerolipid with R3=phosphate'), "
               "('P(OCC[N+](C)(C)C)(OC[C@H](OC(=O)CCCCCCCCCCCCCC)COC(=O)CCCCCCCCCCCCCCCC)([O-])=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('C([C@@](COC(CCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)=O)(OC(CCCCCCC/C=C\\\\CCCC)=O)[H])O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('[NH3+]CCOP([O-])(=O)OC[C@@H](COC([*])=O)OC([*])=O', 'Valid "
               "1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('C(C(COC(CCCCCCCCCCCCCCC)=O)O/C=C\\\\CCCCCCCC/C=C\\\\CCCCCC)OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('O(C(=O)CCCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CC)[C@H](COC(=O)CCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)CO', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('P(OCC[N+](C)(C)C)(OC[C@H](OC(=O)CCCCCCC/C=C\\\\CCCC)COC(=O)CCCCCCC/C=C\\\\CCCCCCCC)([O-])=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('C([C@@](COC(CCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)=O)(OC(CCCCCCC/C=C\\\\C/C=C\\\\CCCCC)=O)[H])OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('P(OCC[N+](C)(C)C)(OC[C@H](O)COC(=O)CC)([O-])=O', 'Valid "
               "1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('O(C[C@](COP([O-])(=O)OCC[N+](C)(C)C)(OC(CCCCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC)=O)[H])CCCCCCCCCCCC/C=C\\\\CCCCCCCC', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('C(C[NH3+])OP(=O)([O-])OC[C@H](OC(CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C=C\\\\C(CCCCC)O)=O)COC(=O)CCCCCCCCCCCCCCCCC', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('CCCCCCCCCCCCCCCC(=O)O[C@H](COC([*])=O)COP([O-])(=O)OCC(O)COP([O-])(=O)OC[C@H](O)COC([*])=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('[H][C@@](COC(=O)CCCCCCCCCCCCCCC)(COP([O-])([O-])=O)OC(=O)CCCCCCC\\\\C=C/CCCCCC', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phosphate'), "
               "('P(OC[C@@H](CO)OC(=O)CCCCCC)(=O)(OCC[N+](C)(C)C)[O-]', 'Valid "
               "1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('C(CCCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CC)(=O)OCC(COP(=O)(OCC(CO)O)O)OC(CCCCCCCCCCCCCCC)=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('C([C@@](COC(CCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)=O)(OC(CCCCCCC/C=C\\\\C/C=C\\\\CCCCC)=O)[H])O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('O(C(=O)CCCCCCCCCCC)C[C@@H](OC(=O)CC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)CO', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('CCCCCCCCCCCCCCCCCCCC(=O)O[C@H](COC(=O)CCCCCCC\\\\C=C/CCCCCCCC)COP([O-])([O-])=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phosphate'), "
               "('C(\\\\CCCCCCCCOC[C@H](COP([O-])(=O)OCC[NH3+])OC(=O)*)=C\\\\CCCCCCCC', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('O(C[C@](CO)(OC(CCCCCCCCC/C=C\\\\CCCCCC)=O)[H])C(CCCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('C(C[N+](C)(C)C)OP(=O)([O-])OC[C@H](OC(=O)C)COCCCCCCCCCCCCCCC', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('C(C(COC(CCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC)=O)O/C=C\\\\CCCCCCCCCCCCCC)OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('OC[C@](COC(CCCCCCCC1CC2C3C4C5CCC5C4C3C21)=O)([H])OCCCCCCCCC6CCC7C(C6)C8C7C9C8CC9', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('P(OCC[N+](C)(C)C)(OC[C@H](OC(=O)CCCCCCCCCCCCC)COC(=O)CCCCCCC/C=C\\\\CCCCC)([O-])=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('CCCCCCCCCCCCCCCC(=O)OC[C@H](COP([O-])(=O)OCC[N+](C)(C)C)OC(=O)CCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CC', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('CCCCCCCCCCCCCCCCCC(=O)OC[C@H](COP([O-])(=O)O[C@@H]1[C@H](O)[C@H](O)[C@@H](O)[C@H](OP([O-])([O-])=O)[C@H]1O)OC(=O)CCCCCCC\\\\C=C/C\\\\C=C/CCCCC', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('C([C@@](COC(CCCCCCC/C=C\\\\CCCC)=O)(OC(CCCCCCC/C=C\\\\CCCCCC)=O)[H])OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('C([C@@](COC(CCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)=O)(OC(CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)=O)[H])O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('O([C@H](COC(=O)CCCCCCCCCCCCCCC)COP(OCC[N+](C)(C)C)(=O)[O-])C(=O)CCCCCCCCO', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCCCCC)COC(=O)CCCCCCC/C=C\\\\C/C=C\\\\CCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('P(OCC[N+](C)(C)C)(OC[C@H](OC(=O)CCCCCCCCCCCCCCCC)COC(=O)CCCCCCCCCCCCCCCCC)([O-])=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCCCCC)COC(=O)CCCCCCC/C=C\\\\CCCCCCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('O=P([O-])(O[C@@H]1[C@@H]([C@@H]([C@H]([C@@H]([C@H]1O[C@H]2O[C@@H]([C@H]([C@@H]([C@H]2NC(C)=O)O)O)CO)O)O)O)O)OCC(COC(CCCCCCCCCCCCCCCCC)=O)OC(=O)CCCCCCC/C=C\\\\CCCCCCCC', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('CCCCCCCCCCCCCCCC(=O)OC[C@H](COP([O-])(=O)OCCNC(=O)CCCCCCCCC)OC(=O)CCCCCCC\\\\C=C/C\\\\C=C/CCCCC', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('O1C(C(O)C(O)C(O)C1OCC(OC(=O)CCCCCCCCCCCCCCC)COCCCCCCCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)CO', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('C(C[N+](C)(C)C)OP(=O)([O-])OC[C@H](OC(*)=O)COC(=O)*', 'Valid "
               "1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('C([C@@](COC(CCCCCCCCCCCCC)=O)(OC(CCCCCCCCCCCCCCCCCCCCC)=O)[H])O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('P(O[C@H]1[C@H](O[C@H]2OC([C@@H](O)C(O)[C@H]2O)CO)C(O)[C@H](O)C(O)C1O)(OC[C@H](OC(=O)CCCCCCCCCCCCC)COC(=O)CCCCCCCCCCCCCCCCC)(O)=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('O=C(O[C@H](CO[C@H]1O[C@@H]([C@@H](O)C([C@H]1O)O)CO)COCCCCCCCCCCCCCCC)CCCCCCCCCCCCCC', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('C(C[N+](C)(C)C)OP(=O)([O-])OCC(OC(*)=O)COC(=O)*OO', 'Valid "
               "1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('O(C(=O)CCCCCCCCC/C=C\\\\CCCCCCCC)[C@H](COC(=O)CC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)CO', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('CCCCCCCCCCCCCCCC(=O)OC[C@H](COP([O-])(=O)OCC[N+](C)(C)C)OC(=O)CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCC', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('CC(C)[C@H](N)C(=O)OCC(CO)OCn1cnc2c1[nH]c(N)nc2=O', 'Valid "
               "1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('[C@@H](COC(CCCCCCCCCCCCCCC)=O)(COP(OCC[N+](C)(C)C)(=O)[O-])OC(CCC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCCCCC)=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('C([C@@](COC(CCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)=O)(OC(CCCCCCCCCCCCC)=O)[H])OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('C=1(/C=C(/C=C/C=C(/C=C/C2=C(CCCC2(C)C)C)\\\\C)\\\\C)C=C(C=C[N+]1CCOP(OC[C@@H](COC(*)=O)OC(*)=O)(=O)[O-])/C=C/C=C(/C=C/C3=C(CCCC3(C)C)C)\\\\C', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('OC[C@@H](COP([O-])(=O)OC[C@H](CO)OC([*])=O)OC([*])=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('O[C@@H]1[C@@H]([C@H]([C@H](O)C1)/C=C/[C@@H](O)CCCCC)CC(=O)CCCCC(OC(CO)CO)=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('C[N+](C)(C)CCOP([O-])(=O)OC[C@H](O)COC([*])=O', 'Valid "
               "1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('O(C(=O)CCCCCCC/C=C\\\\CCCC)[C@H](COC(=O)CCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)CO', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H'), "
               "('P(OCC[N+](C)(C)C)(OC[C@H](OC(=O)CCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)COC(=O)CCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)([O-])=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('C([C@](COC(CCCCCCCCCCC/C=C\\\\CCCCCCCC)=O)([H])O)OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=phospholipid'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCCC)COC(=O)CCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid 1,2-diacyl-sn-glycerolipid with R3=H')]\n"
               "False negatives: [('O(*)C[C@@H](CO*)OC(*)=O', 'R3 position "
               'does not match allowed groups (H, phosphate, or '
               "phospholipid)')]",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 100,
    'num_true_negatives': 2107,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0196078431372549,
    'recall': 1.0,
    'f1': 0.038461538461538464,
    'accuracy': 0.9547306473517428}